/* 
  Copyright (c) 1997-2003 Gavin E. Crooks <gec@compbio.berkeley.edu> 
		     Univ. of California, Berkeley

  Permission is hereby granted, free of charge, to any person obtaining a 
  copy of this software and associated documentation files (the "Software"),
  to deal in the Software without restriction, including without limitation
  the rights to use, copy, modify, merge, publish, distribute, sublicense,
  and/or sell copies of the Software, and to permit persons to whom the
  Software is furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
  THE SOFTWARE.

  (This is the MIT Open Source License, 
  http://www.opensource.org/licenses/mit-license.html)

*/


/* *******************************************************************
   Computational Biology Toolkit - Utilities

   $Id: cbt_util.h,v 1.1 2003/08/19 03:55:47 gec Exp $
******************************************************************** */


#ifndef __CBT_UTIL_H__
#define __CBT_UTIL_H__

//! Intended for internal use. Taken from GSL
#define __CBT_DBL_EPSILON__ (.2204460492503131e-16)


const double cbt_bits_per_nat;

// return the index of the minimal value of vec
size_t cbt_argmin(const size_t len, const int vec[len]);

// return the index of the minimal value of vec 
size_t cbt_argmin_dbl(const size_t len, const double vec[len]);

// return the index of the maximal value of vec 
size_t cbt_argmax(const size_t len, const int vec[len]);

// return the index of the maximal value of vec 
size_t cbt_argmax_dbl(const size_t len, const double vec[len]);

// Divide each element of vec by the total of vec.
// Return the prenormalized total of vec.
double cbt_normalize(const size_t len, double vec[len]);

// Generate a random number from the system clock.  
unsigned long int  cbt_ran_clock(void);

// Print an error message and exit with failure. 
void cbt_die(const char *comment, ...);

// Divide each element of vec by the total of vec.
// Return the prenormalized total of vec.
double cbt_normalize(const size_t len, double vec[len]);

// Aritmetric mean
double cbt_mean(const size_t len, const double vec[len]);


double 
cbt_variance(const size_t len,
             const double data[]);

double 
cbt_stddev(const size_t len,
           const double data[]); 

double 
cbt_entropy( const size_t len,  
             const double * prob);



#endif /* __GSL_UTIL_H__ */
