
/*
  Extract the amino acid profiles from a HMMER model.
*/


#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "cbt_util.h"
#include "hmmer-io.h"

#define AA_TYPES (20)
#define MAX_SEQ_LEN (1024)
#define MAX_LINE_LEN (1024)
#define INTSCALE (1000.)

#ifndef TRUE
#define TRUE (1)
#endif

#ifndef FALSE
#define FALSE (0)
#endif



aa_profile *  aa_profile_create(void) {
  aa_profile * self;
  self = calloc(1,sizeof(struct aa_profile_str));
  if(self==NULL) cbt_die ("Cannot Allocate space");
  return self;
}

void aa_profile_free(aa_profile * self) {
  if(self != NULL) free(self);
}


int aa_profile_read_hmmer2(aa_profile * self, char * id) {
  char dir[] = "hmmlib/";
  char filename[FILENAME_MAX]="";
  FILE * fhmm;
  int flag;


  // DANGER WILL ROBINSON. Unsafe hack. FIXME
  strcat(filename, dir);
  strcat(filename, id+1);
  strcat(filename, ".hmm");
  
  printf("%s\n", filename);
  fhmm = fopen( filename, "r");
  if(fhmm==NULL) return TRUE; // ERROR

  flag = aa_profile_read_hmmer(self, fhmm);
  fclose(fhmm);
  return flag;

}



int aa_profile_read_hmmer(aa_profile * self, FILE * fin) { 
  int i,s;
  char * flag;
  char buffer[MAX_LINE_LEN];
  const char header[] = "HMMER2.0";
  double total, value;
  double null_model[AA_TYPES];

  do {
    if(fgets(buffer, MAX_LINE_LEN, fin)==NULL)
      return EOF; // End of file
  }
  while(buffer[0]=='#' || buffer[0]=='\n');
  
  if( strncmp(buffer, "//", 2) ==0) {
    // EMPTY HMM
    self->len = 0;
    return 0;
  }


  if( strncmp( buffer, header, 8)) 
    cbt_die("Not a recognized HMM file: %s", buffer);
  
  flag =fgets(buffer, MAX_LINE_LEN, fin);
  if(flag==NULL || strncmp(buffer,"NAME  ",6)!=0) 
    cbt_die("Unexpected error");
  sscanf(buffer+6, "%s", self->name);

  //strncpy(self->name, buffer+6, MAX_LINE_LEN);

  //printf("NAME: %s\n", self->name);


  flag =fgets(buffer, MAX_LINE_LEN, fin);
  if(flag==NULL || strncmp(buffer,"LENG  ",6)!=0) 
    cbt_die("Unexpected error");
  self->len =atoi( buffer+6);
  assert(self->len >0);
  assert(self->len <MAX_SEQ_LEN);

  //printf("LEN: %d\n", self->len);

  do {
    flag =fgets(buffer, MAX_LINE_LEN, fin);
    if(flag==NULL) cbt_die("ARgh");
  }
  while( strncmp(buffer, "NULE  ", 6) !=0);

  for(i=0;i<AA_TYPES; i++) {
    null_model[i] = (double) atoi( buffer+ 6 + i*7);
    // Convert to probability
    null_model[i] = (0.05) * pow(2, null_model[i]/INTSCALE);
    //printf("%g\t", null_model[i]);
  }
  //printf("\n");
  
  total = cbt_normalize( AA_TYPES, null_model);
  // printf("TOTAL: %g\n", total);


  flag =fgets(buffer, MAX_LINE_LEN, fin);

  do {
    flag =fgets(buffer, MAX_LINE_LEN, fin);
    if(flag==NULL) cbt_die("ARgh");
  }
  while( strncmp(buffer, "         m->m", 13) !=0);
  
  if(fgets(buffer, MAX_LINE_LEN, fin)==NULL) 
    cbt_die("Argh");


  for(s=0; s<self->len; s++) {
    flag =fgets(buffer, MAX_LINE_LEN, fin);
    if(flag==NULL) cbt_die("Argh");

    for(i=0;i<AA_TYPES; i++) {
      value = (double) atoi( buffer+ 6 + i*7);
      self->profile[s][i]= 
        null_model[i] * pow(2, value/INTSCALE);
      //printf("%g\t", self->profile[s][i]);
    }
    //printf("\n");

    total = cbt_normalize( AA_TYPES, self->profile[s]);
    //printf("TOTAL: %g\n", total);

    for(i=0;i<2; i++) 
      if(fgets(buffer, MAX_LINE_LEN, fin)==NULL) cbt_die("ARgh");
  }

  // Sometimes there is an extra line?
  //if(fgets(buffer, MAX_LINE_LEN, fin)==NULL) cbt_die("ARgh");
  //if( strncmp(buffer, "//", 2) ==0) return 0;

  if(fgets(buffer, MAX_LINE_LEN, fin)==NULL) cbt_die("ARgh");
  if( strncmp(buffer, "//", 2) !=0) cbt_die("argh here 134: %s", buffer);
  return 0;
}

/*
Int main(int argc, char *argv[]) {
  int flag;
  aa_profile * model;

  model = aa_profile_create();
  
  while(TRUE) {
    flag = aa_profile_hmmer_read(model, stdin);
    if(flag == EOF) break;
    printf("%d\t%s:\n", model->len, model->name);
  }

  exit(0);
}
*/
